// This file is part of Awali.
// Copyright 2016-2020 Sylvain Lombardy, Victor Marsault, Jacques Sakarovitch
//
// Awali is a free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

#ifndef COMMON_HISTORYKIND_HH
#define COMMON_HISTORYKIND_HH

namespace awali {

  /** @brief The different kinds of history. 
   *
   * This class represents the different kinds of history a state may have.
   * For instance, if an automaton is the result of a determinisation, then
   * the history of a state is of kind PARTITION.
   * */
  enum class history_kind_t {

    /** @brief The states comes from a single state. */
    SINGLE, 

    /** @brief The states comes from a tuple of state. 
     * It is typically, the history kind of an automaton resulting from a
     * product of automata.
     */
    TUPLE, 
    
    /** @brief The state comes from a rational expression. */
    RATEXP, 
    
    /** @brief The state history is expressed directy as a string.
     * It is typically the history kind typically used when no other kind is
     * proper. 
     * */
    STRING,
    
    /** @brief The state has no history. */
    NO_HISTORY, 
    
    /** @brief The state is a set of states of another automaton.  
     * It is typically the history kind of an automaton resulting from a
     * determinization process. 
     */
    PARTITION
  };

} 
#endif
