#ifndef AWALI_JSON_CC
#define AWALI_JSON_CC

#include<awali/common/json.hh>

namespace awali {

  const unsigned js_value::OBJECT = 0;
  const unsigned js_value::ARRAY = 1;
  const unsigned js_value::INTEGER = 2;
  const unsigned js_value::FLOATING = 3;
  const unsigned js_value::STRING = 4;
  const unsigned js_value::BOOLEAN = 5;
  const unsigned js_value::_NULL = 6;

//   const std::vector<std::string> js_value::type_names = 
//     { "OBJECT", "ARRAY", "INTEGER", "FLOATING", "BOOLEAN", "_NULL" };
  
  js_object* js_value::object() 
  { throw std::runtime_error("This js_object is not a js_object");}
  
  unsigned js_int::type() const {
    return js_value::INTEGER;
  }


  js_object* js_object::object() {return this;}
  
  unsigned js_float::type() const {
    return js_value::FLOATING;
  }
  
  js_value* js_float::parse(std::istream& i) {
    double y;
    i>>y;
    if(std::floor(y)==y)
return new js_int((int)y);
    return new js_float(y);      
  }

  unsigned js_cst::type() const {
    if(value==-1)
      return js_value::_NULL;
    else
      return js_value::BOOLEAN;
  }
    
  js_cst* js_cst::parse(std::istream& i) {
    static const char* v[]={"null","false","true"};
    char c;
    int k,l=0;
    i>> c;
    switch(c) {
      case 'n' : k=0; break;
      case 'f' : k=1; break;
      case 't' : k=2; break;
      default : k=3;
    }
    if(k<3)
      for(l=1; v[k][l]!='\0'; l++) {
        i>>c;
        if(v[k][l]!=c)
          break;
      }
    if(k==3 || v[k][l]!='\0')
      throw std::runtime_error("json parsing");
    return new js_cst(k-1);
  }

  unsigned js_string::type() const {
    return js_value::STRING;
  }
  
  js_string* js_string::parse(std::istream& i) {
    return new js_string(sttc::parsestring(i));
  }
  
  unsigned js_array::type() const {
    return js_value::ARRAY;
  }
    
  js_array* js_array::parse(std::istream& i) {
    js_array* a=new js_array();
    sttc::check(i,'[');
    if(sttc::peek(i)==']')
      return a;
    while(1) {
      a->values.emplace_back(js_value::parse(i));
      if(sttc::peek(i)!=',')
        break;
      sttc::check(i,',');
    }
    sttc::check(i,']');
    return a;
  }
    
  js_array::~js_array() {
    for(js_value* p : values)
      delete p;
  } 
  
    
  bool js_object::has_field(const std::string& key) const {
    return fields.find(key)!= fields.end();
  }
  
  js_value* js_object::value(const std::string& key) {
    return fields.find(key)->second;
  }
  
  js_object* js_object::object(const std::string& key) {
    return value(key)->object();
  }
  
  std::vector<js_value*>& js_object::vect(const std::string& key) {
     return dynamic_cast<js_array*>(value(key))->values;
   }

  int js_object::intg(const std::string& key) {
    js_int * ji=dynamic_cast<js_int*>(value(key));
    if(ji!=NULL)
      return ji->value;
    js_string * js=dynamic_cast<js_string*>(value(key));
    return std::stoi(js->value);
   }

  std::string& js_object::str(const std::string& key) {
    return dynamic_cast<js_string*>(value(key))->value;
  }
  
  unsigned js_object::type() const {
    return js_value::OBJECT;
  }
  
  js_object* js_object::parse(std::istream& i) {
    js_object* o=new js_object();
    sttc::check(i,'{');
    if(sttc::peek(i)=='}')
      return o;
    while(1) {
      std::string k(sttc::parsestring(i));
      sttc::check(i,':');
      js_value* child = js_value::parse(i);
      o->fields.emplace(std::make_pair(k,child));
      if(sttc::peek(i)!=',')
        break;
      sttc::check(i,',');
    }
    sttc::check(i,'}');
    return o;
  }
  
  js_object::~js_object() {
    for(auto p : fields)
      delete p.second;
  } 

  js_value* js_value::parse(std::istream& i) {
    char c;
    switch(c=sttc::peek(i)) {
    case '{' : return js_object::parse(i);
    case '[' : return js_array::parse(i);
    case '"' : return js_string::parse(i);
    default :
      if((c>='0' && c<='9') || c=='-' || c=='.')
        return js_float::parse(i);
      else
        return js_cst::parse(i);
    }
  }

}//namespace awali


#endif
