// This file is part of Awali.
// Copyright 2016-2020 Sylvain Lombardy, Victor Marsault, Jacques Sakarovitch
//
// Awali is a free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

#ifndef DYN_MODULES_PROPER_CC
#define DYN_MODULES_PROPER_CC

#include <unordered_map>
#include <awali/dyn/core/automaton.hh>
#include <awali/dyn/core/abstract-ratexp.hh>
#include <awali/dyn/loading/handler.hh>

#include <awali/dyn/modules/proper.hh>

namespace awali { namespace dyn {
  void proper_here(automaton_t aut, param_t dir, bool prune) {
    loading::call1<void>("proper_here", "proper", aut, dir, prune);
  }

  automaton_t proper(automaton_t aut, param_t dir, bool prune) {
    return loading::call1<automaton_t>("proper", "proper", aut, dir, prune);
  }

  bool is_proper(automaton_t aut) {
    return loading::call1<bool>("is_proper", "proper", aut);
  }

  bool is_valid(automaton_t aut) {
    return loading::call1<bool>("is_valid", "proper", aut);
  }

  automaton_t allow_eps_transition(automaton_t aut) {
    return loading::call1<automaton_t>("allow_eps_transition", "proper", aut);
  }

  automaton_t thompson(ratexp_t exp) {
    return loading::call1<automaton_t>("thompson", "proper", exp);
  }
    
  automaton_t allow_words(automaton_t aut) {
    return loading::call1<automaton_t>("allow_words", "proper", aut);
  }

  automaton_t letterize(automaton_t aut) {
    return loading::call1<automaton_t>("letterize", "proper", aut);
  }

  automaton_t compact(automaton_t aut) {
    return loading::call1<automaton_t>("compact", "proper", aut);
  }


  automaton_t zpc(ratexp_t exp) {
    return loading::call1<automaton_t>("zpc", "proper", exp);
  }

  automaton_t draw_exp(ratexp_t exp) {
    return loading::call1<automaton_t>("draw_exp", "proper", exp);
  }
}}//end of ns awali::dyn

#endif
