// This file is part of Awali.
// Copyright 2016-2019 Sylvain Lombardy, Victor Marsault, Jacques Sakarovitch
//
// Awali is a free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

#ifndef AWALI_ALGOS_FINITE_IMAGE_HH
#define AWALI_ALGOS_FINITE_IMAGE_HH

#include <awali/sttc/algos/proper.hh>
#include <awali/sttc/algos/is-acyclic.hh>
#include <awali/sttc/labelset/traits.hh>
#include <awali/sttc/misc/add-epsilon-trans.hh> // is_epsilon

namespace awali {
  namespace sttc {
    
    template<unsigned I, typename Tdc>
    bool is_of_finite_image(const Tdc& tdc) {
      using labelset_t = labelset_t_of<Tdc>;
      using Ilabelset_t = typename labelset_t::template valueset_t<I>;
      auto ptdc = proper(tdc);
      for(auto tr : ptdc->transitions()) {
	if(!is_epsilon<Ilabelset_t>(std::get<I>(ptdc->label_of(tr))))
	  ptdc->del_transition(tr);
      }
      return is_acyclic(ptdc);
    }
  }
}//end of ns awali::stc

#endif
