// This file is part of Awali.
// Copyright 2016-2020 Sylvain Lombardy, Victor Marsault, Jacques Sakarovitch
//
// Awali is a free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

#ifndef AWALI_MISC_JSON_HH
#define AWALI_MISC_JSON_HH

#include<iostream>
#include<sstream>

#define __SUPPRESS_UNUSED_WARNING__(a) (void)a;


namespace awali { namespace sttc {

  static inline
  std::ostream& tabs(std::ostream& o, int n) {
    __SUPPRESS_UNUSED_WARNING__(tabs)
    for(int i=0; i<n; i++)
      o << "  ";
    return o;
  }

  static inline
  std::ostream& js_formate(std::ostream& o, std::istream& i) {
    __SUPPRESS_UNUSED_WARNING__(js_formate)
    int depth=0;
    char c;
    bool inopen=false;
    bool instring=false;
    while(! i.eof()) {
      i.get(c);
      switch(c) {
      case '\t': break;
      case '{':
      if(!instring) {
        inopen=true;
        if(depth>0)
          o << std::endl;
        tabs(o,depth++);
      }
        o << c;
        break;
      case '}':
      if(!instring) {
        --depth;
        if(depth<0)
          return o;
        if(!inopen){
          o << std::endl;
          tabs(o,depth);
        }
        inopen=false;
      }
      o << c;
	break;
      case '\n' :
	o << std::endl << ' ';
	tabs(o,depth-1);
	break;
      case '"' :
        instring=!instring;
      default :
        o << c;
      }
    }
    return o;
  }

  static inline
  void check(std::istream& i, char e) {
    __SUPPRESS_UNUSED_WARNING__(check)
    static char buffer[]="parser . <> .";
    buffer[7]=e;
    char c;
    do {
      i>>c;
    }while(c==' ' || c== '\n' || c== '\t');
    //std::cerr << c << '&' << std::endl;
    if(c!=e){
      buffer[12]=c;
      throw std::runtime_error(buffer);
    }
  }

  static inline
  char peek(std::istream& i) {
    __SUPPRESS_UNUSED_WARNING__(peek)
    char c;
    while(1) {
      c=i.peek();
      if(c!=' ' && c!= '\n' && c!= '\t')
        return c;
      i.get(c);
    }
  }


  static inline
  std::string parsestring(std::istream& i) {
    __SUPPRESS_UNUSED_WARNING__(parsestring)
    char c;
    i >> c;
    if(c!='"')
      throw std::runtime_error("parser \"");
    std::ostringstream o;
    while(true) {
      i.get(c);
      if(c=='"')
        break;
      o << c;
      if(c=='\\') {
        i.get(c);
        o << c;
      }
    }
    //std::cout << '>' << o.str() << '<' << std::endl;
    return o.str();
  }

  static inline
  char parsecst(std::istream& i) {
    __SUPPRESS_UNUSED_WARNING__(parsecst)
    std::string s;
    int l;
    switch(i.peek()) {
    case 'n' : case 't': l=4; break;
    case 'f' : l=5; break;
    default:
      throw std::runtime_error("parser cst");
    }
    for(int n=0; n<l; ++n) {
      char c;
      i.get(c) ;
      s.append(1,c);
    }
    if(s!="true" && s!="none"  && s!="null" && s!= "false")
      throw std::runtime_error("parser cst");
    return s[0];
  }

  static inline
  int parseint(std::istream& i) {
    __SUPPRESS_UNUSED_WARNING__(parseint)
    int x;
    i >> x;
    return x;
  }

  static inline
  std::string get_first_attr(std::istream& i) {
    __SUPPRESS_UNUSED_WARNING__(get_first_attr)
    check(i, '{');
    std::string attr = parsestring(i);
    check(i, ':');
    return attr;
  }
  
  static inline
  void parseignore(std::istream& i) {
    __SUPPRESS_UNUSED_WARNING__(parseignore)
    switch(peek(i)) {
    case '"':
      parsestring(i);
      if(peek(i)!=':')
        return;
      check(i,':');
      parseignore(i);
      return;
    case '{':
      check(i,'{');
      while(peek(i)!='}')
        parseignore(i);
      check(i,'}');
      return;
    case '[' :
      check(i,'[');
      while(1) {
        parseignore(i);
        if(peek(i)!=',')
          break;
        check(i,',');
      }
      check(i,']');
      return;
    case 't': case 'n' : case 'f':
      parsecst(i);
      return;
    default :
      throw std::runtime_error("Parse ignore");
    }
  }

}}//end of ns awali::stc

#undef __SUPPRESS_UNUSED_WARNING__
#endif                 //AWALI_MISC_JSON_HH

